package mirrg.swing.neon.v1_1;

import java.awt.Window;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.WindowConstants;

import mirrg.event.nitrogen.api.v1_0.INitrogenEventManager;
import mirrg.event.nitrogen.v2_1.HNitrogenEvent;
import mirrg.swing.neon.v1_1.artifacts.logging.HLog;

public class Neon
{

	private INitrogenEventManager eventManager;
	private Window window;

	public Neon(JFrame window)
	{
		this(window, HNitrogenEvent.createInstance());
	}

	public Neon(JDialog window)
	{
		this(window, HNitrogenEvent.createInstance());
	}

	public Neon(JFrame window, INitrogenEventManager eventManager)
	{
		this((Window) window, eventManager);
	}

	public Neon(JDialog window, INitrogenEventManager eventManager)
	{
		this((Window) window, eventManager);
	}

	private Neon(Window window, INitrogenEventManager eventManager)
	{
		this.window = window;
		this.eventManager = eventManager;
		init();
	}

	public INitrogenEventManager getEventManager()
	{
		return eventManager;
	}

	private volatile boolean isInitialized = false;

	public void init()
	{
		window.addComponentListener(new ComponentListener() {

			@Override
			public void componentShown(ComponentEvent e)
			{
				visible = true;

				if (!isInitialized) {
					isInitialized = true;
					eventManager.post(new NitrogenEventNeon.Initialized(window, Neon.this, e));
				}
				eventManager.post(new NitrogenEventNeon.Shown(window, Neon.this, e));
			}

			@Override
			public void componentResized(ComponentEvent e)
			{

			}

			@Override
			public void componentMoved(ComponentEvent e)
			{

			}

			@Override
			public void componentHidden(ComponentEvent e)
			{
				visible = false;

				eventManager.post(new NitrogenEventNeon.Hidden(window, Neon.this, e));
			}

		});
		window.addWindowListener(new WindowListener() {

			@Override
			public void windowOpened(WindowEvent e)
			{

			}

			@Override
			public void windowIconified(WindowEvent e)
			{

			}

			@Override
			public void windowDeiconified(WindowEvent e)
			{

			}

			@Override
			public void windowDeactivated(WindowEvent e)
			{

			}

			@Override
			public void windowClosing(WindowEvent e)
			{

			}

			@Override
			public void windowClosed(WindowEvent e)
			{
				isDisposed = true;

				if (visible) {
					eventManager.post(new NitrogenEventNeon.Hidden(window, Neon.this, e));
				}
				eventManager.post(new NitrogenEventNeon.Disposed(window, Neon.this, e));
			}

			@Override
			public void windowActivated(WindowEvent e)
			{

			}

		});
	}

	public void prepareFrame(int defaultCloseOperation)
	{
		if (window instanceof JFrame) {
			((JFrame) window).setDefaultCloseOperation(defaultCloseOperation);
		} else if (window instanceof JDialog) {
			((JDialog) window).setDefaultCloseOperation(defaultCloseOperation);
		} else {
			HLog.processExceptionWarning(
				new ClassCastException(window.toString() + " is not a JFrame or JDialog."));
		}

		window.pack();
		window.setLocationByPlatform(true);
	}

	public void prepareFrame()
	{
		prepareFrame(WindowConstants.DISPOSE_ON_CLOSE);
	}

	////////////////////////////////////////////////

	private boolean visible = false;

	private volatile boolean isDisposed = false;

	/**
	 * スレッドセーフ
	 */
	public boolean isDisposed()
	{
		return isDisposed;
	}

}
